@@ -799,3 +799,308 @@ end
     e3 = graph2.merged3.parent
     @test e3.inputs == [graph1.a1, graph2.a2]
 end
+
+@testset "compute_identity" begin
+    graph1 = ComputeGraph()
+    add_input!(graph1, :a1, Ref{Any}(1))
+    map!(x -> Ref{Any}(x), graph1, :a1, :b1)
+
+    graph2 = ComputeGraph()
+    add_input!(graph2, :b1, graph1.b1)
+    graph2.b1[]
+    @test graph2.b1.value isa Ref{Any}
+
+    edge = graph2.b1.parent
+    @test edge.callback == ComputePipeline.compute_identity
+    @test length(edge.inputs) == length(edge.outputs)
+    for (in, out) in zip(edge.inputs, edge.outputs)
+        @test in.value === out.value
+        @test !ComputePipeline.isdirty(in)
+        @test !ComputePipeline.isdirty(out)
+    end
+
+    update!(graph1, a1 = 5.0)
+
+    for (in, out) in zip(edge.inputs, edge.outputs)
+        @test in.value === out.value
+        @test ComputePipeline.isdirty(in)
+        @test ComputePipeline.isdirty(out)
+    end
+
+    graph1.b1[]
+
+    for (in, out) in zip(edge.inputs, edge.outputs)
+        @test in.value === out.value
+        @test !ComputePipeline.isdirty(in)
+        @test ComputePipeline.isdirty(out)
+    end
+
+    graph2.b1[]
+
+    for (in, out) in zip(edge.inputs, edge.outputs)
+        @test in.value === out.value
+        @test !ComputePipeline.isdirty(in)
+        @test !ComputePipeline.isdirty(out)
+    end
+end
+
+@testset "is_same NaN and missing" begin
+    # vectors
+    v = [NaN, missing, 1]
+    @test ComputePipeline.is_same(v, copy(v))
+    @test !ComputePipeline.is_same(v, v)
+
+    # Dicts with NaN
+    d = Dict(:a => NaN, :b => missing, :c => 1)
+    @test ComputePipeline.is_same(d, copy(d))
+    @test !ComputePipeline.is_same(d, d)
+end
+
+@testset "map_latest!" begin
+    @testset "Basic functionality" begin
+        graph = ComputeGraph()
+        add_input!(graph, :input, 1)
+
+        # Simple computation with single input and output
+        map_latest!(graph, [:input], [:output]) do x
+            return (x * 2,)
+        end
+
+        # First call should block until result is ready
+        @test graph[:output][] == 2
+
+        # Update and verify it computes the new value
+        update!(graph, input = 5)
+        graph[:output][] # poll
+        sleep(0.1)
+        # Force resolution to pick up the new value
+        @test graph[:output][] == 10
+    end
+
+    @testset "Multiple inputs and outputs" begin
+        graph = ComputeGraph()
+        add_input!(graph, :a, 1)
+        add_input!(graph, :b, 2)
+
+        map_latest!(graph, [:a, :b], [:sum, :product]) do a, b
+            return (a + b, a * b)
+        end
+
+        @test graph[:sum][] == 3
+        @test graph[:product][] == 2
+
+        update!(graph, a = 3, b = 4)
+        graph[:sum][]  # poll
+        sleep(0.3)
+        # Check results - these will force resolution
+        @test graph[:sum][] == 7
+        @test graph[:product][] == 12
+    end
+
+    @testset "Skips intermediate updates" begin
+        graph = ComputeGraph()
+        add_input!(graph, :input, 0)
+
+        computation_count = Ref(0)
+        processed_values = Int[]
+
+        map_latest!(graph, [:input], [:output]) do x
+            computation_count[] += 1
+            push!(processed_values, x)
+            sleep(0.2)  # Simulate slow computation
+            return (x * 2,)
+        end
+
+        # Initialize
+        @test graph[:output][] == 0
+        @test computation_count[] == 1
+
+        # Rapidly update multiple times
+        for i in 1:5
+            update!(graph, input = i)
+        end
+
+        # Wait for computation to complete
+        graph[:output][]  # poll
+        sleep(0.5)
+
+        # Access the result - this forces resolution
+        result = graph[:output][]
+
+        # Should have processed initial + final value(s), but not all intermediate ones
+        @test computation_count[] == 2  # Less than initial + all 5 updates
+        @test result == 10  # Final value (5 * 2)
+    end
+
+    @testset "Returns previous value while computing" begin
+        graph = ComputeGraph()
+        add_input!(graph, :input, 1)
+
+        map_latest!(graph, [:input], [:output]) do x
+            sleep(0.3)  # Simulate slow computation
+            return (x * 10,)
+        end
+
+        # Get initial value
+        @test graph[:output][] == 10
+
+        # Update with new value
+        update!(graph, input = 2)
+
+        # Immediately check - should return previous value while computing
+        prev_value = graph[:output][]
+        @test prev_value == 10  # Still the old value
+
+        # Wait for computation to finish
+        sleep(0.6)
+
+        # Now should have new value
+        @test graph[:output][] == 20
+    end
+
+    @testset "spawn parameter" begin
+        graph = ComputeGraph()
+        add_input!(graph, :input, 5)
+
+        # Test with spawn=false (runs on current task)
+        map_latest!(graph, [:input], [:output]; spawn = false) do x
+            return (x + 1,)
+        end
+
+        @test graph[:output][] == 6
+
+        update!(graph, input = 10)
+        graph[:output][] # poll
+        sleep(0.2)
+        @test graph[:output][] == 11
+    end
+
+    @testset "Computation updates trigger correctly" begin
+        graph = ComputeGraph()
+        add_input!(graph, :input, 1)
+
+        update_count = Ref(0)
+
+        map_latest!(graph, [:input], [:output]) do x
+            sleep(0.05)
+            return (x * 3,)
+        end
+
+        # Initialize
+        @test graph[:output][] == 3
+
+        # Set up an observable listener to count updates
+        obs = ComputePipeline.get_observable!(graph, :output)
+        on(obs) do val
+            update_count[] += 1
+        end
+
+        # Update input
+        update!(graph, input = 2)
+        sleep(0.3)
+
+        # Should have triggered at least one update
+        @test update_count[] >= 1
+        @test graph[:output][] == 6
+    end
+end
+
+@testset "explicit node initialization" begin
+    calls = 0
+    graph = ComputeGraph()
+    add_input!(graph, :x, 1)
+    add_input!(graph, :y, 2)
+    map!(graph, [:x, :y], [:xy, :yx]) do x, y
+        calls += 1
+        return (x + y, x - y)
+    end
+    map!(x -> 2x, graph, :xy, :z)
+
+    @testset "Initial State" begin
+        @test isdirty(graph.x)
+        @test isdirty(graph.y)
+        @test isdirty(graph.xy)
+        @test isdirty(graph.yx)
+        @test isdirty(graph.z)
+    end
+
+    @testset "Partial output init doesn't init edge" begin
+        ComputePipeline.unsafe_init!(graph.xy, 1)
+
+        @test isdirty(graph.x)
+        @test isdirty(graph.y)
+        @test isdirty(graph.xy)
+        @test isdirty(graph.yx)
+        @test isdirty(graph.z)
+        @test ComputePipeline.is_initialized(graph.xy)
+        @test !ComputePipeline.is_initialized(graph.yx)
+        @test graph.xy.value[] == 1
+        @test !isassigned(graph.xy.parent.typed_edge)
+        @test calls == 0
+    end
+
+    @testset "Partial output init doesn't break normal init" begin
+        @test graph.z[] == 6
+        @test calls == 1
+    end
+
+    calls = 0
+    graph = ComputeGraph()
+    add_input!(graph, :x, 1)
+    add_input!(graph, :y, 2)
+    map!(graph, [:x, :y], [:xy, :yx]) do x, y
+        calls += 1
+        return (x + y, x - y)
+    end
+    map!(x -> 2x, graph, :xy, :z)
+    ComputePipeline.unsafe_init!(graph.xy, 1)
+
+    @testset "Verify reset" begin
+        @test isdirty(graph.x)
+        @test isdirty(graph.y)
+        @test isdirty(graph.xy)
+        @test isdirty(graph.yx)
+        @test isdirty(graph.z)
+        @test ComputePipeline.is_initialized(graph.xy)
+        @test !ComputePipeline.is_initialized(graph.yx)
+        @test graph.xy.value[] == 1
+        @test !isassigned(graph.xy.parent.typed_edge)
+        @test calls == 0
+    end
+
+    @testset "Full output init does init edge" begin
+        ComputePipeline.unsafe_init!(graph.yx, Base.Ref{Any}(2))
+
+        @test !isdirty(graph.x)
+        @test !isdirty(graph.y)
+        @test !isdirty(graph.xy)
+        @test !isdirty(graph.yx)
+        @test isdirty(graph.z)
+        @test ComputePipeline.is_initialized(graph.xy)
+        @test ComputePipeline.is_initialized(graph.yx)
+        @test isassigned(graph.xy.parent.typed_edge)
+        @test calls == 0
+        @test graph.xy.value[] == 1
+        @test graph.yx.value[] == 2
+        @test calls == 0
+        @test !ComputePipeline.is_initialized(graph.z)
+        @test graph.z[] == 2
+        @test calls == 0
+    end
+
+    @testset "updates still work" begin
+        graph.x = 2
+        @test isdirty(graph.x)
+        @test !isdirty(graph.y)
+        @test isdirty(graph.xy)
+        @test isdirty(graph.z)
+        @test calls == 0
+        @test graph.xy[] == 4
+        @test graph.z[] == 8
+        @test calls == 1
+    end
+
+    @testset "error on double init" begin
+        @test_throws ErrorException ComputePipeline.unsafe_init!(graph.xy, 0)
+    end
+end
